// Server Class

#ifndef SERVER_H_INCLUDED
#define SERVER_H_INCLUDED

#include <windows.h>

#include "lstnsock.h"

// So CServer knows about CUser
class CUser;

class CServer
{
public:

    // Boot all users, and reset the singleton
    ~CServer();

    // Call GetServer whenever you want the server.
    // Either it will get the server or make a new one.
    // This is a simple singleton - a way to have a global with only classes
    // The effect is that there can only ever be one CServer class
    static inline CServer* GetServer()
    {
        if(theserver==NULL)
        {
            theserver=new CServer;
        }
        return theserver;
    }

    // The server socket
    CListenSocket socket;

    // Handles any socket related messages from the windows message stream
    void SocketMessageHandler(WPARAM wParam, LPARAM lParam);

    // Keep a linked list of online users
    CUser* rootuser;

    // Add a user to the list (becomes the new root node)
    CUser* AddUser();
    CUser* AddUser(CUser* user);

    // Serch for a user by socket
    CUser* GetUser(SOCKET sock);

    // Clear the user list
    void RemoveAllUsers();

private:

    // This is a singleton - we don't want anything other than CServer to call the constructor
    CServer();

    // A pointer to the one instance of the server
    static CServer* theserver;

};

#endif // SERVER_H_INCLUDED
